@@ -3,6 +3,7 @@ import os.path
 import traceback
 from types import SimpleNamespace as namespace
 from urllib.parse import urlparse, urljoin
+import concurrent.futures
 
 import numpy as np
 from AnyQt.QtCore import Qt, QTimer, QThread, QThreadPool
@@ -16,21 +17,13 @@ from Orange.widgets.gui import widgetBox, widgetLabel, comboBox, auto_commit
 from Orange.widgets.settings import Setting
 from Orange.widgets.utils import concurrent as qconcurrent
 from Orange.widgets.utils.itemmodels import VariableListModel
-from Orange.widgets.widget import OWWidget, Default
+from Orange.widgets.widget import Input, Output
+from Orange.widgets.widget import OWWidget
 
 from orangecontrib.imageanalytics.image_embedder import ImageEmbedder
 from orangecontrib.imageanalytics.image_embedder import MODELS as EMBEDDERS_INFO
 
 
-class _Input:
-    IMAGES = 'Images'
-
-
-class _Output:
-    EMBEDDINGS = 'Embeddings'
-    SKIPPED_IMAGES = 'Skipped Images'
-
-
 class OWImageEmbedding(OWWidget):
     name = "Image Embedding"
     description = "Image embedding through deep neural networks."
@@ -40,11 +33,12 @@ class OWImageEmbedding(OWWidget):
     want_main_area = False
     _auto_apply = Setting(default=True)
 
-    inputs = [(_Input.IMAGES, Table, 'set_data')]
-    outputs = [
-        (_Output.EMBEDDINGS, Table, Default),
-        (_Output.SKIPPED_IMAGES, Table)
-    ]
+    class Inputs:
+        images = Input('Images', Table)
+
+    class Outputs:
+        embeddings = Output('Embeddings', Table, default=True)
+        skipped_images = Output('Skipped Images', Table)
 
     cb_image_attr_current_id = Setting(default=0)
     cb_embedder_current_id = Setting(default=0)
@@ -61,9 +55,7 @@ class OWImageEmbedding(OWWidget):
         self._task = None
         self._setup_layout()
         self._image_embedder = None
-        self._executor = qconcurrent.ThreadExecutor(
-            self, threadPool=QThreadPool(maxThreadCount=1)
-        )
+        self._executor = concurrent.futures.ThreadPoolExecutor(max_workers=1)
         self.setBlocking(True)
         QTimer.singleShot(0, self._init_server_connection)
 
@@ -93,8 +85,10 @@ class OWImageEmbedding(OWWidget):
             orientation=Qt.Horizontal,
             callback=self._cb_embedder_changed
         )
-        self.cb_embedder.setModel(VariableListModel(
-            [EMBEDDERS_INFO[e]['name'] for e in self.embedders]))
+        names = [EMBEDDERS_INFO[e]['name'] +
+                 (" (local)" if EMBEDDERS_INFO[e].get("is_local") else "")
+                 for e in self.embedders]
+        self.cb_embedder.setModel(VariableListModel(names))
         if not self.cb_embedder_current_id < len(self.embedders):
             self.cb_embedder_current_id = 0
         self.cb_embedder.setCurrentIndex(self.cb_embedder_current_id)
@@ -132,11 +126,12 @@ class OWImageEmbedding(OWWidget):
             self._image_embedder.is_connected_to_server()
         )
 
+    @Inputs.images
     def set_data(self, data):
         if not data:
             self._input_data = None
-            self.send(_Output.EMBEDDINGS, None)
-            self.send(_Output.SKIPPED_IMAGES, None)
+            self.Outputs.embeddings.send(None)
+            self.Outputs.skipped_images.send(None)
             self.input_data_info.setText(self._NO_DATA_INFO_TEXT)
             return
 
@@ -179,6 +174,7 @@ class OWImageEmbedding(OWWidget):
             self.commit()
         else:
             self.input_data_info.setText(self._NO_DATA_INFO_TEXT)
+        self._set_server_info(self._image_embedder.is_connected_to_server())
 
     def commit(self):
         if self._task is not None:
@@ -189,8 +185,8 @@ class OWImageEmbedding(OWWidget):
             return
 
         if not self._image_attributes or self._input_data is None:
-            self.send(_Output.EMBEDDINGS, None)
-            self.send(_Output.SKIPPED_IMAGES, None)
+            self.Outputs.embeddings.send(None)
+            self.Outputs.skipped_images.send(None)
             return
 
         self._set_server_info(connected=True)
@@ -229,7 +225,7 @@ class OWImageEmbedding(OWWidget):
         def cancel():
             task.future.cancel()
             task.cancelled = True
-            task.embedder.cancelled = True
+            task.embedder.set_canceled(True)
 
         embedder = self._image_embedder
 
@@ -286,15 +282,16 @@ class OWImageEmbedding(OWWidget):
             embeddings = f.result()
         except ConnectionError:
             self._log.exception("Error", exc_info=True)
-            self.send(_Output.EMBEDDINGS, None)
-            self.send(_Output.SKIPPED_IMAGES, None)
+            self.Outputs.embeddings.send(None)
+            self.Outputs.skipped_images.send(None)
             self._set_server_info(connected=False)
             return
         except Exception as err:
             self._log.exception("Error", exc_info=True)
-            self.error("\n".join(traceback.format_exception_only(type(err), err)))
-            self.send(_Output.EMBEDDINGS, None)
-            self.send(_Output.SKIPPED_IMAGES, None)
+            self.error(
+                "\n".join(traceback.format_exception_only(type(err), err)))
+            self.Outputs.embeddings.send(None)
+            self.Outputs.skipped_images.send(None)
             return
 
         assert self._input_data is not None
@@ -312,8 +309,8 @@ class OWImageEmbedding(OWWidget):
     def _send_output_signals(self, embeddings):
         embedded_images, skipped_images, num_skipped =\
             ImageEmbedder.prepare_output_data(self._input_data, embeddings)
-        self.send(_Output.SKIPPED_IMAGES, skipped_images)
-        self.send(_Output.EMBEDDINGS, embedded_images)
+        self.Outputs.embeddings.send(embedded_images)
+        self.Outputs.skipped_images.send(skipped_images)
         if num_skipped is not 0:
             self.input_data_info.setText(
                 "Data with {:d} instances, {:d} images skipped.".format(
@@ -321,10 +318,15 @@ class OWImageEmbedding(OWWidget):
 
     def _set_server_info(self, connected):
         self.clear_messages()
+        if self._image_embedder is None:
+            return
+
         if connected:
             self.connection_info.setText("Connected to server.")
+        elif self._image_embedder.is_local_embedder():
+            self.connection_info.setText("Using local embedder.")
         else:
-            self.connection_info.setText("No connection with server.")
+            self.connection_info.setText("Not connected to server.")
             self.warning("Click Apply to try again.")
 
     def onDeleteWidget(self):
@@ -349,7 +351,7 @@ class OWImageEmbedding(OWWidget):
             self.setBlocking(False)
             self.cb_image_attr.setDisabled(False)
             self.cb_embedder.setDisabled(False)
-            self._image_embedder.cancelled = False
+            self._image_embedder.set_canceled(False)
             # reset the connection.
             connected = self._image_embedder.reconnect_to_server()
             self._set_server_info(connected=connected)
